#include <tgvoip/VoIPController.h>
#include <tgvoip/VoIPServerConfig.h>

#include <CLI11.hpp>
#include <chrono>
#include <fstream>
#include <iostream>
#include <thread>

#include "TgVoIPControllerWrapper.hpp"
#include "WavHelpers.hpp"
#include "utils.hpp"

int main(int argc, char *argv[])
{
    ::setvbuf(stdout, nullptr, _IOLBF, BUFSIZ);
    CLI::App app;

    // tgvoipcall reflector:port tag_caller_hex -k encryption_key_hex -i /path/to/sound_A.ogg -o /path/to/sound_output_B.ogg -c config.json -r caller
    std::string reflector_host_port, tag_caller_hex, tag_callee_hex, encryption_key_hex,
        input_file_name = "input.ogg", output_file_name = "output.ogg", config_file_name = "config.json", role_name = "caller", proxy_host_port;
    app.add_option("reflector:port", reflector_host_port)->required();
    app.add_option("tag_caller_hex", tag_caller_hex)->required();
    app.add_option("tag_callee_hex", tag_callee_hex);
    app.add_option("-k,--key", encryption_key_hex, "encryption_key_hex")->required();
    app.add_option("-i,--input-file", input_file_name, "input file");
    app.add_option("-o,--output-file", output_file_name, "output file");
    app.add_option("-c,--config-file", config_file_name, "config file");
    app.add_option("-r,--role", role_name, "role: 'caller' or 'callee'");
    app.add_option("-p,--proxy", proxy_host_port, "proxy_host:port");

    CLI11_PARSE(app, argc, argv)

    std::ifstream config_file{config_file_name};
    std::string   config{std::istreambuf_iterator<char>(config_file), std::istreambuf_iterator<char>()};

    tgvoip::ServerConfig *cfg = tgvoip::ServerConfig::GetSharedInstance();
    cfg->Update(config);

    bool enable_2nd_controller = false;
    if (!tag_callee_hex.empty()) {
        enable_2nd_controller = true;
    }

    bool is_controller1_outgoing = (role_name == "caller");
    if (enable_2nd_controller) {
        is_controller1_outgoing = true;
    }

    if (!enable_2nd_controller) {
        TgVoIPControllerWrapper controller1{reflector_host_port, tag_caller_hex, encryption_key_hex, is_controller1_outgoing};
        controller1.start();
        controller1.connect();

        std::this_thread::sleep_for(std::chrono::milliseconds(15000));

        std::cout << controller1.getDebugLog() << std::endl;
    } else {
        TgVoIPControllerWrapper controller1{reflector_host_port, tag_caller_hex, encryption_key_hex, true, input_file_name, output_file_name};
        controller1.start();
        controller1.connect();

        TgVoIPControllerWrapper controller2{reflector_host_port, tag_callee_hex, encryption_key_hex, false, input_file_name};
        controller2.start();
        controller2.connect();

        std::this_thread::sleep_for(std::chrono::milliseconds(15000));

        std::cout << controller1.getDebugLog() << std::endl;
        std::cout << controller2.getDebugLog() << std::endl;
    }

    return 0;
}
