#pragma once

#include <vector>
#include <string>

static std::pair<std::string, uint16_t> parse_host_port(const std::string &str)
{
    auto        pos  = str.find_last_of(':');
    std::string host = str;
    uint16_t    port = 0;
    if (pos != std::string::npos) {
        host = str.substr(0, pos);
        port = uint16_t(std::stoul(str.substr(pos + 1)));
    }
    return std::make_pair(host, port);
}

static inline uint8_t unhex_char(char c)
{
    if (c >= '0' && c <= '9') {
        return uint8_t(c) - '0';
    } else if (c >= 'A' && c <= 'F') {
        return uint8_t(c) - 'A' + 10;
    } else if (c >= 'a' && c <= 'f') {
        return uint8_t(c) - 'a' + 10;
    }
    return 0;  // throw "I hope I don't need to take care of this case by task...";
}

static std::vector<uint8_t> hex2bin(const std::string &str)
{
    std::vector<uint8_t> result;
    if (str.length() & 1) {
        return result;  // hex string must have even length
    }
    result.resize(str.length() / 2);
    for (size_t i = 0; i < str.length(); i += 2) {
        result[i / 2] = uint8_t(unhex_char(str[i]) << 4) | (unhex_char(str[i + 1]));
    }
    return result;
}
